/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Toggle Word Wrap.
 *
 * The Initial Developer of the Original Code is
 * Kaspar Brand.
 * Portions created by the Initial Developer are Copyright (C) 2006-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* $Id: toggleWordWrap.js 103 2011-02-04 17:58:15Z kbrand $ */

// Reviews from AMO editors now (Aug 2009) insist on using separate namespaces
if (!ch) var ch = {};
if (!ch.velox) ch.velox = {};
if (!ch.velox.tww) ch.velox.tww = {};

window.addEventListener("load", function() {
// based on a sample/idea from Cesar Oliveira (thanks!)

function toggleWordWrap()
{
  if (!gMsgCompose || gMsgCompose.composeHTML)
    return;

  if (gMsgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth > 0) {
    try {
      gMsgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth = 0 
    }
    catch (e) { }
  } else {
    try {
      gMsgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth = gMsgCompose.wrapLength;
    }
    catch (e) { }
  }
}

function checkToggleWordWrapMenuEntry()
{
  if (!gMsgCompose || gMsgCompose.composeHTML) {
    document.getElementById("menu_togglewordwrap")
            .setAttribute('disabled', true);
  } else {
    document.getElementById("menu_togglewordwrap")
            .setAttribute('checked',gMsgCompose.editor
            .QueryInterface(nsIPlaintextEditorMail).wrapWidth == 0 ?
            false : true);
    document.getElementById("menu_togglewordwrap")
            .setAttribute('disabled', false);
  }
}

function checkTogglePreWrapContextMenuEntry()
{
  /* only needed for Thunderbird/Seamonkey - display an entry in the
   * context menu if the message body is shown in HTML mode
   */
  if (gPrefBranch.getPrefType("mailnews.display.prefer_plaintext")
        == gPrefBranch.PREF_BOOL
      && !gPrefBranch.getBoolPref("mailnews.display.prefer_plaintext")) {
    document.getElementById("context-toggleprewrap").setAttribute('hidden', false);
    document.getElementById("context-toggleprewrap").setAttribute('checked',
      window.content.document.getElementById("toggleprewrap") ? true : false);
  } else {
    document.getElementById("context-toggleprewrap").setAttribute('hidden', true);
  }
}

function togglePreWrap()
{
  // if it's Thunderbird in plain text mode, then don't do anything
  if (Components.classes["@mozilla.org/xre/app-info;1"].
                 getService(Components.interfaces.nsIXULAppInfo).ID
                 == "{3550f703-e582-4d05-9a08-453d09bdfdc6}"
      && gPrefBranch.getPrefType("mailnews.display.prefer_plaintext")
         == gPrefBranch.PREF_BOOL
      && gPrefBranch.getBoolPref("mailnews.display.prefer_plaintext")) return;

  var elem = window.content.document.getElementById("toggleprewrap");
  var mtpw = document.getElementById("menu_toggleprewrap");
  if (elem) {
    elem.parentNode.removeChild(elem);
    refreshTableDisplay();
    /* for Thunderbird, we only add a context menu,
     * so the "menu_toggleprewrap" element might not exist
     */
    if (mtpw) mtpw.setAttribute('checked', false);
  } else {
    var headElem = window.content.document.getElementsByTagName("head")[0];
    if (!headElem) {
      if (mtpw) mtpw.setAttribute('disabled', true);
      document.getElementById("context-toggleprewrap").setAttribute('disabled', true);
      return;
    }
    elem = window.content.document.createElement("style");
    elem.id = "toggleprewrap";
    elem.type = "text/css";
    elem.innerHTML = "pre { ";
    var gv = geckoGetRv();
    if (gv < 1.09) {
      // before Gecko 1.9, there's only -moz-pre-wrap
      elem.innerHTML += "white-space: -moz-pre-wrap !important;";
    } else {
      // Gecko 1.9 then changed to pre-wrap (and 1.9.1 finally dropped support for -moz-pre-wrap)
      elem.innerHTML += "white-space: pre-wrap !important;";
    }
    if (gv > 1.09) {
      // 1.9.1 and later also support the "word-wrap" property
      elem.innerHTML += " word-wrap: break-word !important;"
    }
    elem.innerHTML += " } /* inserted by the Toggle Word Wrap extension */";
    headElem.appendChild(elem);
    refreshTableDisplay();
    if (mtpw) mtpw.setAttribute('checked', true);
  }
}

function refreshTableDisplay()
{
  var table = window.content.document.getElementsByTagName("table");
  if (table) {
    for (var i = 0; i < table.length; i++) {
      // trick Firefox into rewrapping the table
      var temprow = table[i].insertRow(-1);
      table[i].deleteRow(-1);
    }
  }
}

function checkTogglePreWrapMenuEntry()
{
  var mtpw = document.getElementById("menu_toggleprewrap");
  var ctpw = document.getElementById("context-toggleprewrap");

  /* we don't want to show the menu entry in the
   * mail-related "View" menus
   */
  if (/messenger(compose)?\.xul$/.test(document.location)) {
    if (mtpw) mtpw.setAttribute('hidden', true);
    return;
  }

  // figure out if PRE wrapping is currently active or not
  var elem = window.content.document.getElementById("toggleprewrap");
  if (elem) {
    if (mtpw) mtpw.setAttribute('checked', true);
    if (ctpw) ctpw.setAttribute('checked', true);
  } else {
    if (mtpw) mtpw.setAttribute('checked', false);
    if (ctpw) ctpw.setAttribute('checked', false);
  }
  // if there's no head element, then we can't do anything useful... disable the item
  if (window.content.document.getElementsByTagName("head")[0]) {
    if (mtpw) mtpw.setAttribute('disabled', false);
    if (ctpw) ctpw.setAttribute('disabled', false);
  } else {
    if (mtpw) mtpw.setAttribute('disabled', true);
    if (ctpw) ctpw.setAttribute('disabled', true);
  }
}

function geckoGetRv()
{
  /* see also https://developer.mozilla.org/En/Browser_Detection_and_Cross_Browser_Support
   *
   * this gets us the "pure" Gecko version, even if the
   * "general.useragent.override" pref is set (which is also returned
   * by navigator.userAgent). In theory, nsIHttpProtocolHandler
   * allows changing the value of the "misc" attribute; in practice,
   * however, it's highly unlikely that someone has done so.
   */
  var rvParts = Components.classes["@mozilla.org/network/protocol;1?name=http"]
                          .getService(Components.interfaces.nsIHttpProtocolHandler)
                          .misc.substr(3).split('.');
  var rvValue = 0;
  var exp     = 1;

  for (var i = 0; i < rvParts.length; i++)
  {
    var val = parseInt(rvParts[i]);
    rvValue += val / exp;
    exp *= 100;
  }
  return rvValue;
}

/* Replace GenericSendMessage() with a wrapper which makes sure that
 * "format=flowed" is not added when word wrapping has been disabled.
 * nsMsgCompUtils.cpp would do this even if wrapWidth has been set to 0
 * (and a message body with lines of more than 78 characters - something
 * which SHOULDn't really happen according to RFC 3676, but anyway...).
 * We therefore disable mailnews.send_plaintext_flowed temporarily, where
 * applicable.
 */
if (typeof GenericSendMessage == "function")
{
  var _GenericSendMessage = GenericSendMessage;
  GenericSendMessage = function(msgType)
  {
    const kTextFlowed = "mailnews.send_plaintext_flowed";
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
                          .getService(Components.interfaces.nsIPrefBranch);

    if (gMsgCompose && !gMsgCompose.composeHTML
        && !gMsgCompose.editor.QueryInterface(nsIPlaintextEditorMail).wrapWidth
        && prefs.getBoolPref(kTextFlowed)) {
      var cs = Components.classes["@mozilla.org/consoleservice;1"].
                          getService(Components.interfaces.nsIConsoleService);
      cs.logStringMessage("Toggle Word Wrap: temporarily disabling "
                          + kTextFlowed);
      prefs.setBoolPref(kTextFlowed, false);
      _GenericSendMessage(msgType);
      prefs.setBoolPref(kTextFlowed, true);
      cs.logStringMessage("Toggle Word Wrap: " + kTextFlowed + " re-enabled");
    } else {
      _GenericSendMessage(msgType);
    }
  }
}

/* add our checkXYZ() functions to the relevant menus
 * format of the entries:  "menu_id : name_of_handler"
 */
var cmds = {
  // Thunderbird and Seamonkey
  optionsMenuPopup: checkToggleWordWrapMenuEntry,
  // Thunderbird up to version 2
  messagePaneContext: checkTogglePreWrapContextMenuEntry,
  // Thunderbird 2 and later, Seamonkey 2 and later (message window)
  mailContext: checkTogglePreWrapContextMenuEntry,
  // Firefox
  menu_viewPopup: checkTogglePreWrapMenuEntry,
  // Seamonkey (browser window)
  menu_View_Popup: checkTogglePreWrapMenuEntry,
  // Firefox and Seamonkey (browser window)
  contentAreaContextMenu: checkTogglePreWrapMenuEntry
};
for (var id in cmds) {
  var elem = document.getElementById(id);
  if (elem)
    elem.addEventListener("popupshowing", cmds[id], true);
}

// functions we reference in toggleWordWrap.xul
ch.velox.tww.toggleWordWrap = toggleWordWrap;
ch.velox.tww.togglePreWrap = togglePreWrap;

}, false);
